﻿/// <Copyright>
///****************************************************************
///                              NOTICE
///
///      CONFIDENTIAL UNPUBLISHED PROPERTY OF UNITED PARCEL
///      SERVICE. USE AND DISTRIBUTION LIMITED SOLELY TO
///      AUTHORIZED PERSONNEL.
///
///
///      The use, disclosure, reproduction, modification,
///      transfer, or transmittal of this work for any purpose
///      in any form or by any means without the written
///      permission of United Parcel Service is strictly
///      prohibited.
///
///      @Copyright 2020 United Parcel Service of America, Inc.
///                       All Rights Reserved
///
///                   DO NOT REPRODUCE OR DUPLICATE
///*****************************************************************
/// </Copyright>

using System; 
using System.Data; 
using System.Linq;
using System.Windows.Forms; 
using Infragistics.Documents.Excel;
using Infragistics.Win; 
using Infragistics.Win.UltraWinGrid; 

namespace ExportExcelErrorTest
{
    public partial class ViolationReport : Form
    {
        private DataTable dataTable;

        private enum Columns
        {
            OrigID,
            OrigCC,
            OrigRegion,
            OrigDistrict,
            OrigAbbrev,
            OrigSLIC,
            OrigSort,
            ServiceID,
            OperationType,
            OriginalPieces,
            OriginalWeight,
            OriginalCube,
            OriginalUtilPieces,
            OriginalUtilWeight,
            OriginalUtilCube,
            CurrentPieces,
            CurrentWeight,
            CurrentCube,
            CurrentUtilPieces,
            CurrentUtilWeight,
            CurrentUtilCube,
            DifferencePieces,
            DifferenceWeight,
            DifferenceCube,
            TotalCapacity,
            StartTime,
            SpanTime,
            CapPerHour,
            OrigBSLIC,
            OrigState
        }


        private readonly string workbookFileName = @"../../HubCapViolation.xlsx";

        #region Constructor
        public ViolationReport()
        {
            InitializeComponent();
        }

        #endregion

        private void SetUpGrid()
        {
            this.SetupDataTable();
            this.SetupData();
        }

        private void SetupData()
        {
            Workbook wk = Workbook.Load(workbookFileName);

            Worksheet ws = wk?.Worksheets[0];
            if (ws == null)
            {
                return;
            }

            for (int rowIndex = 2; rowIndex < ws.Rows.Count(); rowIndex++)
            {
                DataRow dr = this.dataTable.NewRow();
                dr[Columns.OrigID.ToString()] = rowIndex;
                dr[Columns.OrigCC.ToString()] = ws.Rows[rowIndex].Cells[0].Value;
                dr[Columns.OrigRegion.ToString()] = ws.Rows[rowIndex].Cells[1].Value;
                dr[Columns.OrigDistrict.ToString()] = ws.Rows[rowIndex].Cells[2].Value;
                dr[Columns.OrigAbbrev.ToString()] = ws.Rows[rowIndex].Cells[3].Value;
                dr[Columns.OrigSLIC.ToString()] = ws.Rows[rowIndex].Cells[4].Value;
                dr[Columns.OrigSort.ToString()] = ws.Rows[rowIndex].Cells[5].Value;
                dr[Columns.OperationType.ToString()] = ws.Rows[rowIndex].Cells[6].Value;
                dr[Columns.StartTime.ToString()] = ws.Rows[rowIndex].Cells[7].Value;
                dr[Columns.SpanTime.ToString()] = ws.Rows[rowIndex].Cells[8].Value;
                dr[Columns.CapPerHour.ToString()] = ws.Rows[rowIndex].Cells[9].Value;
                dr[Columns.TotalCapacity.ToString()] = ws.Rows[rowIndex].Cells[10].Value;
                dr[Columns.OriginalPieces.ToString()] = ws.Rows[rowIndex].Cells[11].Value;
                dr[Columns.OriginalUtilPieces.ToString()] = ws.Rows[rowIndex].Cells[12].Value;
                dr[Columns.CurrentPieces.ToString()] = ws.Rows[rowIndex].Cells[13].Value;
                dr[Columns.CurrentUtilPieces.ToString()] = ws.Rows[rowIndex].Cells[14].Value;
                dr[Columns.DifferencePieces.ToString()] = ws.Rows[rowIndex].Cells[15].Value;
                dataTable.Rows.Add(dr);
            }

            this.grid.DataSource = this.dataTable;
        }

        private void SetupDataTable()
        {
            this.dataTable = new DataTable("Violation Report");

            this.dataTable.Columns.Add(Columns.OrigID.ToString(), typeof(uint));
            this.dataTable.Columns.Add(Columns.OrigCC.ToString(), typeof(string));
            this.dataTable.Columns.Add(Columns.OrigRegion.ToString(), typeof(string));
            this.dataTable.Columns.Add(Columns.OrigDistrict.ToString(), typeof(string));
            this.dataTable.Columns.Add(Columns.OrigAbbrev.ToString(), typeof(string));
            this.dataTable.Columns.Add(Columns.OrigSLIC.ToString(), typeof(string));
            this.dataTable.Columns.Add(Columns.OrigBSLIC.ToString(), typeof(string));
            this.dataTable.Columns.Add(Columns.OrigState.ToString(), typeof(string));
            this.dataTable.Columns.Add(Columns.OperationType.ToString(), typeof(string));
            this.dataTable.Columns.Add(Columns.OrigSort.ToString(), typeof(string));
            this.dataTable.Columns.Add(Columns.StartTime.ToString(), typeof(short)).AllowDBNull = true;
            this.dataTable.Columns.Add(Columns.SpanTime.ToString(), typeof(double));
            this.dataTable.Columns.Add(Columns.CapPerHour.ToString(), typeof(long));
            this.dataTable.Columns.Add(Columns.TotalCapacity.ToString(), typeof(long));
            this.dataTable.Columns.Add(Columns.OriginalPieces.ToString(), typeof(long));
            this.dataTable.Columns.Add(Columns.OriginalWeight.ToString(), typeof(long));
            this.dataTable.Columns.Add(Columns.OriginalCube.ToString(), typeof(long));
            this.dataTable.Columns.Add(Columns.OriginalUtilPieces.ToString(), typeof(float));
            this.dataTable.Columns.Add(Columns.OriginalUtilWeight.ToString(), typeof(float));
            this.dataTable.Columns.Add(Columns.OriginalUtilCube.ToString(), typeof(float));
            this.dataTable.Columns.Add(Columns.CurrentPieces.ToString(), typeof(long));
            this.dataTable.Columns.Add(Columns.CurrentWeight.ToString(), typeof(long));
            this.dataTable.Columns.Add(Columns.CurrentCube.ToString(), typeof(long));
            this.dataTable.Columns.Add(Columns.CurrentUtilPieces.ToString(), typeof(float));
            this.dataTable.Columns.Add(Columns.CurrentUtilWeight.ToString(), typeof(float));
            this.dataTable.Columns.Add(Columns.CurrentUtilCube.ToString(), typeof(float));
            this.dataTable.Columns.Add(Columns.DifferencePieces.ToString(), typeof(long));
            this.dataTable.Columns.Add(Columns.DifferenceWeight.ToString(), typeof(long));
            this.dataTable.Columns.Add(Columns.DifferenceCube.ToString(), typeof(long));

            DataColumn[] primaryKeys = new DataColumn[1];
            primaryKeys[0] = this.dataTable.Columns[Columns.OrigID.ToString()];
            this.dataTable.PrimaryKey = primaryKeys;
        }


        private void ViolationReportForm_Shown(object sender, EventArgs e)
        {
            SetUpGrid();
            this.grid.DisplayLayout.Override.AllowMultiCellOperations = AllowMultiCellOperation.All;

            // Show primary volume columns and hide other volume type columns - Default
            UpdateVolumeColumns(grid, true, false, false);
            grid.DisplayLayout.Bands[0].Columns[Columns.OrigID.ToString()].ExcludeFromColumnChooser = ExcludeFromColumnChooser.True;

            // Update grid's group
            UpdateGrouping(grid);
            // Update grid's filter evaluation trigger 
            grid.DisplayLayout.Override.FilterEvaluationTrigger = FilterEvaluationTrigger.Default;


            // Set up volume grid data
            this.SetUpGrid();
            foreach (UltraGridColumn col in this.grid.DisplayLayout.Bands[0].Columns)
            {
                if (col.Key.Contains("Util"))
                {
                    col.CellAppearance.TextHAlign = HAlign.Right;
                    col.Format = "#0%";
                    col.AllowRowSummaries = AllowRowSummaries.True;
                }
                else if (col.Key.Contains("Pieces") || col.Key.Contains("Weight"))
                {
                    col.CellAppearance.TextHAlign = HAlign.Right;
                    col.Format = "#,#0";
                    col.AllowRowSummaries = AllowRowSummaries.True;
                }

                if (col.Key.Equals("Start Time"))
                {
                    col.Format = "00:00";
                }

                if (col.Key.Equals("Span Time"))
                {
                    col.Format = "#0.00";
                }
            }

            // Show the default cursor
            this.Cursor = Cursors.Default;
        }



        public static void UpdateGrouping(UltraGrid Grid)
        {
            foreach (UltraGridBand band in Grid.DisplayLayout.Bands)
            {
                foreach (UltraGridGroup group in band.Groups)
                {
                    int groupWidth = 0;
                    bool showGrp = false;
                    foreach (UltraGridColumn col in group.Columns)
                    {
                        if (!col.Hidden)
                        {
                            showGrp = true;
                            groupWidth += col.Width;
                        }
                    }

                    group.Hidden = !showGrp;
                    group.Width = groupWidth;
                }
            }

        }


        private void UpdateVolumeColumns(UltraGrid Grid, bool UsePieces, bool UseWeight, bool UseCube)
        {
            foreach (UltraGridBand band in Grid.DisplayLayout.Bands)
            {
                foreach (UltraGridColumn column in band.Columns)
                {
                    // Pieces columns
                    if (column.Key.Contains("Pieces"))
                    {
                        if (UsePieces && column.Hidden)
                        {
                            continue;
                        }

                        column.Hidden = !UsePieces;
                    }
                    else if (column.Key.Contains("Weight"))  
                    {
                        if (UseWeight && column.Hidden)
                        {
                            continue;
                        }

                        column.Hidden = !UseWeight;
                    }
                    else if (column.Key.Contains("Cube"))  
                    {
                        if (UseCube && column.Hidden)
                        {
                            continue;
                        }

                        column.Hidden = !UseCube;
                    }
                }
            }
        }

        private void excelExporter_InitializeColumn(object sender, Infragistics.Win.UltraWinGrid.ExcelExport.InitializeColumnEventArgs e)
        {
            e.ExcelFormatStr = e.Column.Format;
        }

        private void btnExport_Click(object sender, EventArgs e)
        {
            if (excelSaveFileDialog.ShowDialog() == DialogResult.OK)
            {
                // Create a workbook
                Workbook workbook = new Workbook(WorkbookFormat.Excel2007);
                // Add a worksheet to the workbook
                Worksheet worksheet = workbook.Worksheets.Add(grid.Text);
                // Export the grid to the worksheet
                this.excelExporter.Export(this.grid, worksheet);
                // Save the workbook
                workbook.Save(this.excelSaveFileDialog.FileName);
            }
        }
    } 
}
